-- FuBar_AlarmClockFu 0.5
-- Author: Anthony White
-- Purpose: A simple alarm clock for FuBar
-- See Changelog.txt for revision information

AlarmClockFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

local FMT_MIL = 0 -- 24 hour format
local FMT_NORM = 1 -- 12 hour format
local DISPLAY_LOCAL = 0 -- display local time
local DISPLAY_SERVER = 1 -- display server time
local DISPLAY_BOTH = 2 -- display both times

AlarmClockFu.hour = 0
AlarmClockFu.minute = 0
AlarmClockFu.lastalarmmin = 0

-- Register the saved variable DB
AlarmClockFu:RegisterDB("AlarmClockFuDB")

-- Realm defaults for saved variables
AlarmClockFu:RegisterDefaults('profile', {
	offset = 0, -- the offset from server to local time
	hourfmt = FMT_NORM,
	alarmenable = false,
	repeat_alarm = true,
	alarmenable_2 = false,
	repeat_alarm2 = true,
	alarmmin = 0,
	alarmmin_2 = 0,
	alarmhr = 0,
	alarmhr_2 = 0,
	alarmmsg = "Alarm Activated!",
	alarmmsg_2 = "Alarm 2 Activated!",
	display = DISPLAY_LOCAL
})

-- Menu Request
AlarmClockFu.OnMenuRequest = {
	type = 'group',
	args = {
		offset = {
			type = 'range',
			name = "Offset",
			desc = "Time offset from server time to local time",
			get = "GetOffset",
			set = "SetOffset",
			min = -12,
			max = 12,
			step = 1
		},
		hourfmt = {
			type = 'group',
			desc = "Select a time format",
			name = "Hour Format",
			args = {
				normalfmt = {
					type = 'toggle',
					name = "12 Hour Format",
					desc = "Select a 12 hour time format",
					set = "SetNormalFormat",
					get = "IsNormalFormat"
				},
				militaryfmt = {
					type = 'toggle',
					name = "24 Hour Format",
					desc = "Select a 24 hour time format",
					set = "SetMilitaryFormat",
					get = "IsMilitaryFormat"
				}
			}
		},
		display = {
			type = 'group',
			name = "Display Settings",
			desc = "Choose the time that will be displayed",
			args = {
				disp_local = {
					type = 'toggle',
					name = "Local Time",
					desc = "Display only local time",
					get = "IsLocalFormat",
					set = "SetLocalFormat"
				},
				disp_server = {
					type = 'toggle',
					name = "Server Time",
					desc = "Display only server time",
					get = "IsServerFormat",
					set = "SetServerFormat"
				},
				disp_both = {
					type = 'toggle',
					name = "Show Both",
					desc = "Display both times",
					get = "IsBothFormat",
					set = "SetBothFormat"
				}
			}
		},
		alarm = {
			type = 'group',
			name = "Alarm Settings",
			desc = "Change alarm settings",
			args = {
				alarmenable = {
					type = 'toggle',
					name = "Enable Alarm",
					desc = "Switch the alarm on or off",
					get = "IsAlarmEnabled",
					set = "SetAlarmEnabled"
				},
				repeat_alarm = {
					type = 'toggle',
					name = "Recurring Alarm",
					desc = "Toggle repeating this alarm",
					get = "IsAlarmRepeating",
					set = "SetAlarmRepeating"
				},
				alarmtime = {
					type = 'group',
					name = "Alarm Time",
					desc = "Set the alarm time",
					args = {
						alarmhr = {
							type = 'text',
							name = "Alarm Hour",
							desc = "Set the alarm hour",
							usage = "<hour>",
							get = "GetAlarmHour",
							set = "SetAlarmHour"
						},
						alarmmin = {
							type = 'text',
							name = "Alarm Minute",
							desc = "Set the alarm minute",
							usage = "<minute>",
							get = "GetAlarmMinute",
							set = "SetAlarmMinute"
						}
					}
				},
				alarmmsg = {
					type = 'text',
					name = "Alarm Message",
					desc = "Set the message displayed when the alarm goes off",
					usage = "<alarm message>",
					get = "GetAlarmMessage",
					set = "SetAlarmMessage"
				}
			}
		},
		alarm_2 = {
			type = 'group',
			name = "Alarm 2 Settings",
			desc = "Change alarm settings",
			args = {
				alarmenable = {
					type = 'toggle',
					name = "Enable Alarm",
					desc = "Switch the alarm on or off",
					get = "IsAlarm2Enabled",
					set = "SetAlarm2Enabled"
				},
				repeat_alarm = {
					type = 'toggle',
					name = "Recurring Alarm",
					desc = "Toggle repeating this alarm",
					get = "IsAlarm2Repeating",
					set = "SetAlarm2Repeating"
				},
				alarmtime = {
					type = 'group',
					name = "Alarm Time",
					desc = "Set the alarm time",
					args = {
						alarmhr = {
							type = 'text',
							name = "Alarm Hour",
							desc = "Set the alarm hour",
							usage = "<hour>",
							get = "GetAlarm2Hour",
							set = "SetAlarm2Hour"
						},
						alarmmin = {
							type = 'text',
							name = "Alarm Minute",
							desc = "Set the alarm minute",
							usage = "<minute>",
							get = "GetAlarm2Minute",
							set = "SetAlarm2Minute"
						}
					}
				},
				alarmmsg = {
					type = 'text',
					name = "Alarm Message",
					desc = "Set the message displayed when the alarm goes off",
					usage = "<alarm message>",
					get = "GetAlarm2Message",
					set = "SetAlarm2Message"
				}
			}
		}
	}
}

-- Get / Set Methods
function AlarmClockFu:IsLocalFormat()
	if (self.db.profile.display == DISPLAY_LOCAL) then
		return true
	end
	
	return false
end

function AlarmClockFu:IsServerFormat()
	if (self.db.profile.display == DISPLAY_SERVER) then
		return true
	end
	
	return false
end

function AlarmClockFu:IsBothFormat()
	if (self.db.profile.display == DISPLAY_BOTH) then
		return true
	end
	
	return false
end

function AlarmClockFu:SetLocalFormat()
	self.db.profile.display = DISPLAY_LOCAL
end

function AlarmClockFu:SetServerFormat()
	self.db.profile.display = DISPLAY_SERVER
end

function AlarmClockFu:SetBothFormat()
	self.db.profile.display = DISPLAY_BOTH
end
	
function AlarmClockFu:GetOffset()
	return self.db.profile.offset
end

function AlarmClockFu:SetOffset(v)
	self.db.profile.offset = v
end

function AlarmClockFu:IsMilitaryFormat()
	if self.db.profile.hourfmt == FMT_MIL then
		return true
	end
	
	return false
end

function AlarmClockFu:IsNormalFormat()
	if self.db.profile.hourfmt == FMT_NORM then
		return true
	end
	
	return false
end

function AlarmClockFu:SetMilitaryFormat()
	self.db.profile.hourfmt = FMT_MIL
end

function AlarmClockFu:SetNormalFormat()
	self.db.profile.hourfmt = FMT_NORM
end

function AlarmClockFu:GetAlarmHour()
	return self.db.profile.alarmhr
end

function AlarmClockFu:SetAlarmHour(hour)
	self.db.profile.alarmhr = tonumber(hour)
end

function AlarmClockFu:GetAlarm2Hour()
	return self.db.profile.alarmhr_2
end

function AlarmClockFu:SetAlarm2Hour(hour)
	self.db.profile.alarmhr_2 = tonumber(hour)
end

function AlarmClockFu:GetAlarmMinute()
	return self.db.profile.alarmmin
end

function AlarmClockFu:SetAlarmMinute(min)
	self.db.profile.alarmmin = tonumber(min)
end

function AlarmClockFu:GetAlarm2Minute()
	return self.db.profile.alarmmin_2
end

function AlarmClockFu:SetAlarm2Minute(min)
	self.db.profile.alarmmin_2 = tonumber(min)
end

function AlarmClockFu:IsAlarmEnabled()
	return self.db.profile.alarmenable
end

function AlarmClockFu:SetAlarmEnabled()
	self.db.profile.alarmenable = not self.db.profile.alarmenable
end

function AlarmClockFu:IsAlarmRepeating()
	return self.db.profile.repeat_alarm
end

function AlarmClockFu:SetAlarmRepeating()
	self.db.profile.repeat_alarm = not self.db.profile.repeat_alarm
end

function AlarmClockFu:IsAlarm2Enabled()
	return self.db.profile.alarmenable_2
end

function AlarmClockFu:SetAlarm2Enabled()
	self.db.profile.alarmenable_2 = not self.db.profile.alarmenable_2
end

function AlarmClockFu:IsAlarm2Repeating()
	return self.db.profile.repeat_alarm2
end

function AlarmClockFu:SetAlarm2Repeating()
	self.db.profile.repeat_alarm2 = not self.db.profile.repeat_alarm2
end

function AlarmClockFu:GetAlarmMessage()
	return self.db.profile.alarmmsg
end

function AlarmClockFu:SetAlarmMessage(msg)
	self.db.profile.alarmmsg = msg
end

function AlarmClockFu:GetAlarm2Message()
	return self.db.profile.alarmmsg_2
end

function AlarmClockFu:SetAlarm2Message(msg)
	self.db.profile.alarmmsg_2 = msg
end

-- Formats the text displayed for the addon
function AlarmClockFu:FormatClockString()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour

	hour = hour + self.db.profile.offset
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- used to get the server time
function AlarmClockFu:FormatClockNoOffset()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

-- formats the alarm time
function AlarmClockFu:FormatAlarmString()
	local hour = self.db.profile.alarmhr
	local minute = self.db.profile.alarmmin

	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

function AlarmClockFu:FormatAlarm2String()
	local hour = self.db.profile.alarmhr_2
	local minute = self.db.profile.alarmmin_2

	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime, hour, minute
end

function AlarmClockFu:GetNextMinute(curmin)
	if (curmin == 60) then
		return 0
	else
		return curmin + 1
	end
end

-- Called when the addon is initialized
function AlarmClockFu:OnInitialize()
	self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)
	self:RegisterEvent("AlarmClockFu_AlarmTrigger", self.AlarmClockTrigger, true)
	self:RegisterEvent("AlarmClockFu_AlarmTrigger2", self.AlarmClockTrigger2, true)
end

-- Sets the displayed text
function AlarmClockFu:OnTextUpdate()
	self.hour, self.minute = GetGameTime()
	
	local timestr = ""
	if (self.db.profile.display == DISPLAY_LOCAL) then
		timestr, _, _ = self:FormatClockString()
	elseif (self.db.profile.display == DISPLAY_SERVER) then
		timestr, _, _ = self:FormatClockNoOffset()
	else
		local tempstr = ""
		tempstr, _, _ = self:FormatClockString()
		timestr, _, _ = self:FormatClockNoOffset()
		timestr = tempstr .. " | " .. timestr
	end
	
	self:SetText(timestr)
end

-- Make sure we update the text
-- Also, trigger alarms if need bew
function AlarmClockFu:OnUpdate()
	self:UpdateText()
	
	local oldformat = self.db.profile.hourfmt
	
	if (oldformat == FMT_NORM) then
		self.db.profile.hourfmt = FMT_MIL
	end
	
	local _, hour, min = self:FormatClockString()
	
	self.db.profile.hourfmt = oldformat
	
	if (self:GetNextMinute(self.lastalarmmin) == self.minute) then
		self:RegisterEvent("AlarmClockFu_AlarmTrigger", self.AlarmClockTrigger)
	end
	
	if (self:GetNextMinute(self.lastalarmmin) == self.minute) then
		self:RegisterEvent("AlarmClockFu_AlarmTrigger2", self.AlarmClockTrigger2)
	end
	
	-- is alarm 1 ready?
	if (self.db.profile.alarmenable == true) then
		if (hour == self.db.profile.alarmhr and min == self.db.profile.alarmmin) then
			self:TriggerEvent("AlarmClockFu_AlarmTrigger")
			
			-- check if the alarm is not recurring
			if (self.db.profile.repeat_alarm == false) then
				self.db.profile.alarmenable = false
			end
			
			self.lastalarmmin = self.minute
			if (self:IsEventRegistered("AlarmClockFu_AlarmTrigger")) then
				self:UnregisterEvent("AlarmClockFu_AlarmTrigger")
			end
		end
	end
	
	-- is alarm 2 ready?
	if (self.db.profile.alarmenable_2 == true) then
		if (hour == self.db.profile.alarmhr_2 and min == self.db.profile.alarmmin_2) then
			self:TriggerEvent("AlarmClockFu_AlarmTrigger2")
			self.lastalarmmin = self.minute
			
			-- check if the alarm is not recurring
			if (self.db.profile.repeat_alarm2 == false) then
				self.db.profile.alarmenable_2 = false
			end
			
			if (self:IsEventRegistered("AlarmClockFu_AlarmTrigger2")) then
				self:UnregisterEvent("AlarmClockFu_AlarmTrigger2")
			end
		end
	end
end

-- Event handler for AlarmClockFu_AlarmTrigger event
-- for some reason the self variable is not available here
--function AlarmClockFu:AlarmClockTrigger()  (Commented out by Wiltock)
	--_ERRORMESSAGE(AlarmClockFu.db.profile.alarmmsg)(Commented out by Wiltock)
--end(Commented out by Wiltock)
function AlarmClockFu:AlarmClockTrigger()--Added by Wiltock 
PlaySoundFile("Interface\\AddOns\\FuBar_AlarmClockFu\\bell.wav"); --Added by Wiltock
UIErrorsFrame:AddMessage(AlarmClockFu.db.profile.alarmmsg, 1.0, 0.0, 0.0, 1.0, 3.0);--Added by Wiltock

end
--function AlarmClockFu:AlarmClockTrigger2()(Commented out by Wiltock)
	--_ERRORMESSAGE(AlarmClockFu.db.profile.alarmmsg_2)(Commented out by Wiltock)
--end(Commented out by Wiltock)
function AlarmClockFu:AlarmClockTrigger2() --Added by Wiltock
PlaySoundFile("Interface\\AddOns\\FuBar_AlarmClockFu\\bell.wav");--Added by Wiltock
UIErrorsFrame:AddMessage(AlarmClockFu.db.profile.alarmmsg_2, 1.0, 0.0, 0.0, 1.0, 3.0);--Added by Wiltock

end

local tablet = AceLibrary("Tablet-2.0")
function AlarmClockFu:OnTooltipUpdate()
	local timestr, _, _ = self:FormatClockString()
	local timestr_no_offset, _, _ = self:FormatClockNoOffset()
	local alarmtime, _, _ = self:FormatAlarmString()
	local alarm2time, _, _ = self:FormatAlarm2String()

	-- Add a date line to the tooltip
	-- As suggested by HatteSoul (from ClockFu)
	tablet:AddCategory():AddLine(
		'text', date("%A, %B %d, %Y"),
		'textR', 1,
		'textG', 1,
		'textB', 1,
		'justify', "CENTER"
	)
	
	cat = tablet:AddCategory(
		'text', "Time",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', 1,
		'child_textG2', 1,
		'child_textB2', 1
	)
	
	cat:AddLine(
		'text', "Local Time",
		'text2', timestr
	)
	
	cat:AddLine(
		'text', "Server Time",
		'text2', timestr_no_offset
	)
	
	local offsetstr = ""
	
	if (self.db.profile.offset > 0) then
		offsetstr = "+" .. self.db.profile.offset
	else
		offsetstr = tostring(self.db.profile.offset)
	end
	
	cat:AddLine(
		'text', "Offset",
		'text2', offsetstr
	)
	
	local r, g, b
	
	if (self.db.profile.alarmenable == false) then
		r = 1
		g = 0
		b = 0
	else
		r = 0
		g = 1
		b = 0
	end
	
	cat = tablet:AddCategory(
		'text', "Alarm",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled
	
	if self.db.profile.alarmenable == false then
		enabled = "No"
	else
		enabled = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled
	)
	
	if self.db.profile.alarmenable == true then
		cat:AddLine(
			'text', "Alarm Time",
			'text2', alarmtime
		)
	end
	
	cat = tablet:AddCategory(
		'text', "Alarm 2",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', r,
		'child_textG2', g,
		'child_textB2', b
	)
	
	local enabled_2
	
	if (self.db.profile.alarmenable_2 == false) then
		enabled_2 = "No"
	else
		enabled_2 = "Yes"
	end
	
	cat:AddLine(
		'text', "Enabled",
		'text2', enabled_2
	)
	
	if (self.db.profile.alarmenable_2 == true) then
		cat:AddLine(
			'text', "Alarm 2 Time",
			'text2', alarm2time
		)
	end
end